import 'dart:convert';
import 'package:http/http.dart' as http;
import 'package:modular_system_core/modular_system_core.dart';

class AppController<T extends JsonSerializer<T>> {
  AppController({
    required this.baseUri,
    required this.controllerName,
    required this.objectCreator,
  });

  final String baseUri;
  final String controllerName;
  late final String fullUri = "$baseUri/$controllerName";
  final JsonObjectCreator<T> objectCreator;

  Future<StringDTO> createAsync(T object) async {
    var resp = await http.post(
      Uri.parse(fullUri),
      headers: <String, String>{
        'Content-Type': 'application/json',
      },
      body: object.toJson(),
    );
    var jsonMap = getJsonFrom(resp);
    return StringDTO.fromJson(jsonMap);
  }

  // READ
  Future getByIdAsync(String id) async {
    var resp = await http.get(
      Uri.parse("$fullUri?id=$id"),
      headers: <String, String>{
        'Content-Type': 'application/json; charset=UTF-8',
      },
    );
    var jsonMap = getJsonFrom(resp);
    return objectCreator(jsonMap);
  }

  Future<QueryList<T>> searchAsync(WebSearchRequest searchRequest) async {
    var body = jsonEncode(searchRequest.toJson());
    var resp = await http.post(
      Uri.parse("$fullUri/query"),
      headers: <String, String>{
        'Content-Type': 'application/json; charset=UTF-8',
      },
      body: body,
    );
    var jsonMap = getJsonFrom(resp);
    return QueryList.fromJsonMap(jsonMap, objectCreator);
  }

  // UPDATE

  // DELETE
  Future deleteAsync(String id) async {
    var resp = await http.delete(
      Uri.parse("$fullUri?id=$id"),
      headers: <String, String>{
        'Content-Type': 'application/json',
      },
    );
    handleResponse(resp);
  }

  Map<String, dynamic> getJsonFrom(Response resp) {
    if (resp.statusCode == 200) {
      return jsonDecode(resp.body);
    } else if (resp.body.isNotEmpty) {
      throw AppException.fromJson(resp.body);
    } else {
      throw AppException("${resp.statusCode} ${resp.reasonPhrase}");
    }
  }

  void handleResponse(Response resp) {
    if (resp.statusCode == 200) {
      return;
    } else if (resp.body.isNotEmpty) {
      throw AppException.fromJson(resp.body);
    } else {
      throw AppException("${resp.statusCode} ${resp.reasonPhrase}");
    }
  }
}
