import 'dart:convert';

import 'package:modular_system_core/modular_system_core.dart';

import 'dob.dart';
import 'location.dart';

enum Gender {
  unknown,
  male,
  female,
}

class Person implements JsonSerializer<Person> {
  Person();

  factory Person.fromJson(Map<String, dynamic> jsonMap) {
    var i = Person();
    i.age = jsonMap["age"];
    i.gender = Gender.values[jsonMap["gender"]];
    i.name = PersonName.fromJson(jsonMap["name"]);
    i.dateOfBirth = DateOfBirth.fromJson(jsonMap["dateOfBirth"]);
    return i;
  }

  int age = 0;
  Gender gender = Gender.unknown;
  PersonName name = PersonName();
  DateOfBirth dateOfBirth = DateOfBirth();

  @override
  Person fromJson(Map<String, dynamic> jsonMap) {
    var i = Person();
    i.age = jsonMap["age"];
    i.gender = Gender.values[jsonMap["gender"]];
    i.name = PersonName.fromJson(jsonMap["name"]);
    i.dateOfBirth = DateOfBirth.fromJson(jsonMap["dateOfBirth"]);
    return i;
  }

  @override
  Map<String, dynamic> toJsonMap() {
    var map = {
      'age': age,
      'gender': gender.index,
      'name': name.toJsonMap(),
      'dateOfBirth': dateOfBirth.toJsonMap(),
    };
    return map;
  }

  @override
  String toJson() => jsonEncode(toJsonMap());
}

class PersonName implements JsonSerializer<PersonName> {
  PersonName();

  factory PersonName.fromJson(Map<String, dynamic> jsonMap) {
    var i = PersonName();
    i.firstName = jsonMap["firstName"];
    for (var surname in jsonMap["surnames"]) {
      i.surnames.add(surname);
    }
    return i;
  }

  String firstName = "";
  List<String> surnames = [];

  String getFullName() {
    var name = firstName;
    for (var surname in surnames) {
      name = "$name $surname";
    }
    return name;
  }

  @override
  PersonName fromJson(Map<String, dynamic> jsonMap) {
    var i = PersonName();
    i.firstName = jsonMap["firstName"];
    for (var surname in jsonMap["surnames"]) {
      i.surnames.add(surname);
    }
    return i;
  }

  @override
  Map<String, dynamic> toJsonMap() {
    var map = {
      'firstName': firstName,
      'surnames': surnames,
    };
    return map;
  }

  @override
  String toJson() => jsonEncode(toJsonMap());
}

class Citizen extends Person {
  Citizen();

  factory Citizen.fromJson(Map<String, dynamic> jsonMap) {
    var i = Citizen();
    i.age = jsonMap["age"];
    i.documentNumber = jsonMap["documentNumber"];
    i.gender = Gender.values[jsonMap["gender"]];
    i.name = PersonName.fromJson(jsonMap["name"]);
    i.dateOfBirth = DateOfBirth.fromJson(jsonMap["dateOfBirth"]);
    i.address = Location.fromJson(jsonMap["address"]);
    return i;
  }

  String documentNumber = "";
  Location address = Location();

  @override
  Citizen fromJson(Map<String, dynamic> jsonMap) {
    var i = Citizen();
    i.age = jsonMap["age"];
    i.documentNumber = jsonMap["documentNumber"];
    i.gender = Gender.values[jsonMap["gender"]];
    i.name = PersonName.fromJson(jsonMap["name"]);
    i.dateOfBirth = DateOfBirth.fromJson(jsonMap["dateOfBirth"]);
    i.address = Location.fromJson(jsonMap["address"]);
    return i;
  }

  @override
  Map<String, dynamic> toJsonMap() {
    var map = {
      'age': age,
      'documentNumber': documentNumber,
      'gender': gender.index,
      'name': name.toJsonMap(),
      'dateOfBirth': dateOfBirth.toJsonMap(),
      'address': address.toJsonMap(),
    };
    return map;
  }

  @override
  String toJson() => jsonEncode(toJsonMap());
}
