import 'package:flutter/material.dart';
// ignore: import_of_legacy_library_into_null_safe
import 'package:flutter_icons/flutter_icons.dart';
import 'package:modular_system_core/modular_system_core.dart';
import 'package:modular_system_core/src/app_viewmodel.dart';
import 'viewmodel.dart';

class EntityTableAppBar<T> extends StatefulWidget {
  // ignore: use_key_in_widget_constructors
  const EntityTableAppBar({
    required this.viewModel,
    required this.appViewModel,
    required this.updateTableFuture,
    required this.filtersSheetBuilder,
  });

  final AppViewModel appViewModel;
  final EntityTableViewModel<T> viewModel;
  final Future updateTableFuture;
  final AppBottomSheet Function(
    AppViewModel appViewModel,
    EntityTableViewModel<T> viewModel,
  ) filtersSheetBuilder;

  @override
  State<EntityTableAppBar> createState() {
    // ignore: no_logic_in_create_state
    return _EntityTableAppBarState<T>(
      appViewModel: appViewModel,
      viewModel: viewModel,
      updateTableFuture: updateTableFuture,
      filtersSheetBuilder: filtersSheetBuilder,
    );
  }
}

class _EntityTableAppBarState<T> extends State<EntityTableAppBar> {
  _EntityTableAppBarState({
    required this.viewModel,
    required this.appViewModel,
    required this.updateTableFuture,
    required this.filtersSheetBuilder,
  });

  AppViewModel appViewModel;
  EntityTableViewModel<T> viewModel;
  Future updateTableFuture;
  final AppBottomSheet Function(
    AppViewModel appViewModel,
    EntityTableViewModel<T> viewModel,
  ) filtersSheetBuilder;

  @override
  Widget build(BuildContext context) {
    return SliverAppBar(
      pinned: true,
      elevation: 10,
      backgroundColor: appViewModel.colorScheme.background,
      title: TextField(
        controller: viewModel.searchbarController,
        style: appViewModel.backgroundTextStyle(AppTextType.subtitle),
        decoration: InputDecoration(
          hintText: "Pesquisar ...",
          hintStyle: appViewModel.backgroundTextStyle(AppTextType.subtitle),
          focusedBorder: UnderlineInputBorder(
            borderSide: BorderSide(
              color: appViewModel.colorScheme.onPrimary,
            ),
          ),
          border: InputBorder.none,
        ),
        onChanged: searchbarOnChanged,
      ),
      leading: Icon(
        Icons.search,
        color: appViewModel.colorScheme.onPrimary,
        size: appViewModel.textSize.subtitle,
      ),
      actions: [
        IconButton(
          padding: EdgeInsets.symmetric(
            vertical: 8,
            horizontal: appViewModel.textSize.subtitle * 0.5,
          ),
          icon: Icon(
            Icons.clear,
            color: appViewModel.colorScheme.onPrimary,
            size: appViewModel.textSize.subtitle,
          ),
          onPressed: () {
            viewModel.searchbarController.text = "";
            clearTextFilter();
          },
        ),
        IconButton(
          padding: const EdgeInsets.symmetric(
            vertical: 8,
            horizontal: 10,
          ),
          icon: Icon(
            Icons.post_add,
            color: appViewModel.colorScheme.onPrimary,
            size: appViewModel.textSize.subtitle,
          ),
          onPressed: viewModel.showForm.invoke,
        ),
        IconButton(
          padding: const EdgeInsets.symmetric(
            vertical: 8,
            horizontal: 10,
          ),
          icon: Icon(
            MaterialCommunityIcons.restore,
            color: appViewModel.colorScheme.onPrimary,
            size: appViewModel.textSize.subtitle,
          ),
          onPressed: viewModel.updateEvent.invoke,
        ),
      ],
      bottom: PreferredSize(
        // Add this code
        preferredSize:
            const Size.fromHeight(kToolbarHeight + 10), // Add this code
        child: Padding(
          padding: const EdgeInsets.only(
            bottom: 10,
            left: 5,
            right: 5,
          ),
          child: Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            crossAxisAlignment: CrossAxisAlignment.center,
            children: [
              Row(
                children: [
                  IconButton(
                    icon: Icon(
                      Icons.filter_list,
                      color: appViewModel.colorScheme.onBackground,
                      size: appViewModel.textSize.subtitle,
                    ),
                    onPressed: () {
                      appViewModel.appEvents.bottomSheetEvent
                          .invoke(filtersSheetBuilder(appViewModel, viewModel));
                    },
                  ),
                  VerticalDivider(
                    color: appViewModel.colorScheme.onBackground,
                  ),
                  Text(
                    "Filtros: ${viewModel.search.filters.length}",
                    style:
                        appViewModel.backgroundTextStyle(AppTextType.subtitle),
                  ),
                  VerticalDivider(
                    color: appViewModel.colorScheme.onBackground,
                  ),
                  FutureBuilder(
                    future: updateTableFuture,
                    builder: (context, snapshot) {
                      if (snapshot.connectionState == ConnectionState.done &&
                          !snapshot.hasError) {
                        return Text(
                          "Paginação: ${getPagination()} Total: ${viewModel.paginationOut.total}",
                          style: appViewModel
                              .backgroundTextStyle(AppTextType.subtitle),
                        );
                      } else if (snapshot.hasError) {
                        return Text(
                          "Paginação: ERRO",
                          style: appViewModel
                              .backgroundTextStyle(AppTextType.subtitle),
                        );
                      } else {
                        return Text(
                          "Paginação: CARREGANDO",
                          style: appViewModel
                              .backgroundTextStyle(AppTextType.subtitle),
                        );
                      }
                    },
                  ),
                ],
              ),
              Row(
                mainAxisAlignment: MainAxisAlignment.end,
                children: [
                  FutureBuilder(
                    future: updateTableFuture,
                    builder: (context, snapshot) {
                      if (snapshot.connectionState == ConnectionState.done &&
                          !snapshot.hasError) {
                        return Container(
                          margin: const EdgeInsets.symmetric(
                            horizontal: 25,
                          ),
                          child: Row(
                            children: [
                              IconButton(
                                iconSize: appViewModel.textSize.subtitle,
                                mouseCursor: previousPageExists()
                                    ? null
                                    : SystemMouseCursors.basic,
                                splashRadius: previousPageExists() ? null : 1,
                                color: previousPageExists()
                                    ? appViewModel.colorScheme.tertiary
                                    : appViewModel.colorScheme.onBackground,
                                padding: const EdgeInsets.symmetric(
                                  horizontal: 15,
                                ),
                                icon: const Icon(
                                  Icons.navigate_before,
                                ),
                                onPressed: () {
                                  if (previousPageExists()) {
                                    viewModel.search.pagination.offset =
                                        viewModel.search.pagination.offset -
                                            viewModel.search.pagination.limit;
                                    viewModel.updateEvent.invoke();
                                  }
                                },
                              ),
                              IconButton(
                                iconSize: appViewModel.textSize.subtitle,
                                mouseCursor: nextPageExists()
                                    ? null
                                    : SystemMouseCursors.basic,
                                splashRadius: nextPageExists() ? null : 1,
                                color: nextPageExists()
                                    ? appViewModel.colorScheme.tertiary
                                    : appViewModel.colorScheme.onBackground,
                                padding: const EdgeInsets.symmetric(
                                  horizontal: 15,
                                ),
                                icon: const Icon(
                                  Icons.navigate_next,
                                ),
                                onPressed: () {
                                  if (nextPageExists()) {
                                    viewModel.search.pagination.offset =
                                        viewModel.search.pagination.offset +
                                            viewModel.search.pagination.limit;
                                    viewModel.updateEvent.invoke();
                                  }
                                },
                              ),
                            ],
                          ),
                        );
                      } else if (snapshot.hasError) {
                        return Text(
                          "ERRO",
                          style:
                              appViewModel.errorTextStyle(AppTextType.subtitle),
                        );
                      } else {
                        return Container();
                      }
                    },
                  ),
                  DropdownButton(
                    value: viewModel.search.pagination.limit,
                    focusColor: Colors.transparent,
                    dropdownColor: appViewModel.colorScheme.primary,
                    iconEnabledColor: appViewModel.colorScheme.onBackground,
                    underline: SizedBox(
                      height: 1.5,
                      child: Container(
                        color: appViewModel.colorScheme.onBackground,
                      ),
                    ),
                    items: <DropdownMenuItem<int>>[
                      DropdownMenuItem(
                        value: 10,
                        child: Text(
                          "10",
                          style: appViewModel
                              .backgroundTextStyle(AppTextType.subtitle),
                        ),
                      ),
                      DropdownMenuItem(
                        value: 30,
                        child: Text(
                          "30",
                          style: appViewModel
                              .backgroundTextStyle(AppTextType.subtitle),
                        ),
                      ),
                      DropdownMenuItem(
                        value: 50,
                        child: Text(
                          "50",
                          style: appViewModel
                              .backgroundTextStyle(AppTextType.subtitle),
                        ),
                      ),
                      DropdownMenuItem(
                        value: 100,
                        child: Text(
                          "100",
                          style: appViewModel
                              .backgroundTextStyle(AppTextType.subtitle),
                        ),
                      ),
                    ],
                    onChanged: (limit) {
                      setState(() {
                        viewModel.search.pagination.limit = limit as int;
                        viewModel.updateEvent.invoke();
                      });
                    },
                  ),
                ],
              ),
            ],
          ),
        ), // Add this code
      ),
    );
  }

  String getPagination() {
    var pagination = viewModel.paginationOut;
    var currentPage = pagination.offset / pagination.limit + 1;
    var totalPages = pagination.total / pagination.limit + 1;
    return "$currentPage de ${totalPages.floor()}";
  }

  bool nextPageExists() {
    return viewModel.paginationOut.total >
        (viewModel.paginationOut.offset + viewModel.paginationOut.limit);
  }

  bool previousPageExists() {
    return viewModel.paginationOut.offset > 0;
  }

  void searchbarOnChanged(String value) {
    if (viewModel.search.filters
        .where((element) => element.name == "text")
        .isEmpty) {
      viewModel.search.filters.add(
        WebSearchFilter(
          name: "text",
          type: WebDataType.string,
          isActive: value.isNotEmpty ? true : false,
          value: value,
        ),
      );
    } else {
      viewModel.search.filters
          .where((element) => element.name == "text")
          .first
          .value = value;
      viewModel.search.filters
          .where((element) => element.name == "text")
          .first
          .isActive = value.isNotEmpty ? true : false;
    }
  }

  void clearTextFilter() {
    if (viewModel.search.filters
        .where((element) => element.name == "text")
        .isNotEmpty) {
      viewModel.search.filters.removeWhere((element) => element.name == "text");
      viewModel.search.filters.add(
        WebSearchFilter(
          name: "text",
          type: WebDataType.string,
          value: "",
        ),
      );
    }
  }
}
