import 'package:flutter/material.dart';
import 'package:modular_system_core/modular_system_core.dart';
import 'package:modular_system_core/src/widget/entity_table/table_body.dart';
export 'viewmodel.dart';
export './app_bar.dart';

class EntityTable<T> extends StatefulWidget {
  // ignore: use_key_in_widget_constructors
  const EntityTable({
    required this.appViewModel,
    required this.update,
    required this.itemBuilder,
    required this.formBuilder,
    required this.transitionDuration,
    this.filtersSheetBuilder,
  });

  final AppViewModel appViewModel;
  final Future<SingleEntryMap<PaginationOut, List<T>>> Function(
      WebSearchRequest search) update;
  final Widget Function(BuildContext context, T data, AppViewModel avm,
      EntityTableViewModel<T> vm) itemBuilder;
  final Widget Function(
    BuildContext context,
    AppViewModel appViewModel,
    EntityTableViewModel<T> viewModel,
  ) formBuilder;
  final Duration transitionDuration;
  final AppBottomSheet Function(
    AppViewModel appViewModel,
    EntityTableViewModel<T> viewModel,
  )? filtersSheetBuilder;

  @override
  State<EntityTable> createState() {
    // ignore: no_logic_in_create_state
    return _EntityTableState<T>(
      appViewModel: appViewModel,
      update: update,
      itemBuilder: itemBuilder,
      formBuilder: formBuilder,
      transitionDuration: transitionDuration,
      filtersSheetBuilder:
          filtersSheetBuilder ?? (avm, vm) => AppBottomSheet.empityRounded(),
    );
  }
}

class _EntityTableState<T> extends State<EntityTable> {
  _EntityTableState({
    required this.appViewModel,
    required this.update,
    required this.itemBuilder,
    required this.formBuilder,
    required this.transitionDuration,
    required this.filtersSheetBuilder,
  }) {
    subscribeToEvents();
  }

  @override
  void dispose() {
    unsubscribeToEvents();
    super.dispose();
  }

  final AppViewModel appViewModel;
  final EntityTableViewModel<T> viewModel = EntityTableViewModel();
  final Future<SingleEntryMap<PaginationOut, List<T>>> Function(
      WebSearchRequest search) update;
  final Widget Function(BuildContext context, T data, AppViewModel avm,
      EntityTableViewModel<T> vm) itemBuilder;
  final Widget Function(
    BuildContext context,
    AppViewModel appViewModel,
    EntityTableViewModel<T> viewModel,
  ) formBuilder;
  final Duration transitionDuration;
  final AppBottomSheet Function(
    AppViewModel appViewModel,
    EntityTableViewModel<T> viewModel,
  ) filtersSheetBuilder;
  late Widget body = getTableBody();

  void subscribeToEvents() {
    viewModel.showTable.subscribe(onShowTable);
    viewModel.showForm.subscribe(onShowForm);
  }

  void unsubscribeToEvents() {
    viewModel.showTable.unsubscribe(onShowTable);
    viewModel.showForm.unsubscribe(onShowForm);
  }

  @override
  Widget build(BuildContext context) {
    return AnimatedSwitcher(
      duration: transitionDuration,
      child: body,
    );
  }

  Widget getTableBody() {
    return EntityTableBody(
      appViewModel: appViewModel,
      viewModel: viewModel,
      update: update,
      itemBuilder: itemBuilder,
      formBuilder: formBuilder,
      transitionDuration: transitionDuration,
      filtersSheetBuilder: filtersSheetBuilder,
    );
  }

  void setBody(Widget widget) {
    setState(() {
      body = widget;
    });
  }

  // Event Handlers
  void onShowTable(dynamic data) => setBody(getTableBody());

  void onShowForm(dynamic data) {
    setBody(formBuilder(
      context,
      appViewModel,
      viewModel,
    ));
  }
}
