import 'package:flutter/material.dart';
import 'package:modular_system_core/modular_system_core.dart';

class EntityTableBody<T> extends StatefulWidget {
  // ignore: use_key_in_widget_constructors
  const EntityTableBody({
    required this.appViewModel,
    required this.viewModel,
    required this.update,
    required this.itemBuilder,
    required this.formBuilder,
    required this.transitionDuration,
    required this.filtersSheetBuilder,
  });

  final AppViewModel appViewModel;
  final EntityTableViewModel<T> viewModel;
  final Future<SingleEntryMap<PaginationOut, List<T>>> Function(
      WebSearchRequest search) update;
  final Widget Function(BuildContext context, T data, AppViewModel avm,
      EntityTableViewModel<T> vm) itemBuilder;
  final Widget Function(
    BuildContext context,
    AppViewModel appViewModel,
    EntityTableViewModel<T> viewModel,
  ) formBuilder;
  final Duration transitionDuration;
  final AppBottomSheet Function(
    AppViewModel appViewModel,
    EntityTableViewModel<T> viewModel,
  ) filtersSheetBuilder;

  @override
  State<EntityTableBody> createState() {
    // ignore: no_logic_in_create_state
    return _EntityTableBodyState<T>(
      appViewModel: appViewModel,
      viewModel: viewModel,
      update: update,
      itemBuilder: itemBuilder,
      formBuilder: formBuilder,
      transitionDuration: transitionDuration,
      filtersSheetBuilder: filtersSheetBuilder,
    );
  }
}

class _EntityTableBodyState<T> extends State<EntityTableBody> {
  _EntityTableBodyState({
    required this.appViewModel,
    required this.viewModel,
    required this.update,
    required this.itemBuilder,
    required this.formBuilder,
    required this.transitionDuration,
    required this.filtersSheetBuilder,
  }) {
    subscribeToEvents();
    setUpdateFuture();
  }

  @override
  void dispose() {
    unsubscribeToEvents();
    super.dispose();
  }

  final AppViewModel appViewModel;
  final EntityTableViewModel<T> viewModel;
  final Future<SingleEntryMap<PaginationOut, List<T>>> Function(
      WebSearchRequest search) update;
  final Widget Function(BuildContext context, T data, AppViewModel avm,
      EntityTableViewModel<T> vm) itemBuilder;
  final Widget Function(
    BuildContext context,
    AppViewModel appViewModel,
    EntityTableViewModel<T> viewModel,
  ) formBuilder;
  final Duration transitionDuration;
  final AppBottomSheet Function(
    AppViewModel appViewModel,
    EntityTableViewModel<T> viewModel,
  ) filtersSheetBuilder;
  late Future updateTableFuture;
  late List<Widget> listData = [];

  void subscribeToEvents() {
    viewModel.updateEvent.subscribe(onUpdate);
  }

  void unsubscribeToEvents() {
    viewModel.updateEvent.unsubscribe(onUpdate);
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      color: appViewModel.colorScheme.background,
      child: CustomScrollView(
        slivers: [
          EntityTableAppBar(
            viewModel: viewModel,
            appViewModel: appViewModel,
            updateTableFuture: updateTableFuture,
            filtersSheetBuilder: filtersSheetBuilder,
          ),
          FutureBuilder(
            future: updateTableFuture,
            builder: (context, snapshot) {
              if (snapshot.connectionState == ConnectionState.done) {
                if (snapshot.hasError) {
                  return SliverToBoxAdapter(
                    child: Text(
                      "ERRO",
                      style: appViewModel.errorTextStyle(AppTextType.subtitle),
                    ),
                  );
                }
                return SliverList(
                  key: UniqueKey(),
                  delegate: SliverChildListDelegate(listData),
                );
              }
              return SliverToBoxAdapter(
                child: appViewModel.loading(),
              );
            },
          ),
        ],
      ),
    );
  }

  // Event Handlers
  void onUpdate(dynamic data) {
    setState(() {
      setUpdateFuture();
    });
  }

  void setUpdateFuture() {
    updateTableFuture = Future(() async {
      try {
        var map = await update(viewModel.search);
        viewModel.paginationOut = map.key;
        viewModel.data = map.value;
        setListData();
      } catch (e) {
        appViewModel.appEvents.errorEvent.invoke(e);
      }
    });
  }

  void setListData() {
    listData.clear();
    for (var i = 0; i < viewModel.data.length; i++) {
      listData.add(itemBuilder(
        context,
        viewModel.data[i],
        appViewModel,
        viewModel,
      ));
    }
  }
}
